package org.eclipse.ui.internal;

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IElementFactory;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveRegistry;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.XMLMemento;
import org.eclipse.ui.extensionevents.IExtensionListener;
import org.eclipse.ui.internal.registry.ActionSetPartAssociationsReader;
import org.eclipse.ui.internal.registry.ActionSetRegistry;
import org.eclipse.ui.internal.registry.ActionSetRegistryReader;
import org.eclipse.ui.internal.registry.CapabilityRegistry;
import org.eclipse.ui.internal.registry.CapabilityRegistryReader;
import org.eclipse.ui.internal.registry.EditorRegistry;
import org.eclipse.ui.internal.registry.EditorRegistryReader;
import org.eclipse.ui.internal.registry.IActionSet;
import org.eclipse.ui.internal.registry.IActionSetDescriptor;
import org.eclipse.ui.internal.registry.IViewRegistry;
import org.eclipse.ui.internal.registry.MarkerHelpRegistry;
import org.eclipse.ui.internal.registry.MarkerHelpRegistryReader;
import org.eclipse.ui.internal.registry.MarkerImageProviderRegistry;
import org.eclipse.ui.internal.registry.NewWizardsRegistryReader;
import org.eclipse.ui.internal.registry.PerspectiveRegistry;
import org.eclipse.ui.internal.registry.PerspectiveRegistryReader;
import org.eclipse.ui.internal.registry.ViewRegistry;
import org.eclipse.ui.internal.registry.ViewRegistryReader;
import org.eclipse.ui.internal.registry.WorkingSetRegistry;
import org.eclipse.ui.internal.registry.WorkingSetRegistryReader;

public class ExtensionEventHandler implements IExtensionListener {
	private static final String TAG_CATEGORY="category";//$NON-NLS-1$
	private static final String ATT_TARGET_ID="targetID";//$NON-NLS-1$
	private static final String TAG_PART="part";//$NON-NLS-1$
	private static final String ATT_ID="id";//$NON-NLS-1$
	private static final String TAG_PROVIDER = "imageprovider";//$NON-NLS-1$

	IWorkbench workbench;
	
	public ExtensionEventHandler(IWorkbench workbench) {
		this.workbench = workbench;
	}
	
	public void appear(IExtensionPoint extPt, IExtension ext) {
		String name = extPt.getSimpleIdentifier();
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_VIEWS)) {
			loadView(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_EDITOR)) {
			loadEditor(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_NEW)) {
			NewWizardsRegistryReader.addExtension(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_PERSPECTIVES)) {
			loadPerspective(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_ACTION_SETS)) {
			loadActionSets(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_ACTION_SET_PART_ASSOCIATIONS)) {
			loadActionSetPartAssociation(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_MARKER_IMAGE_PROVIDER)) {
			loadMarkerImageProvider(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_MARKER_HELP)) {
			loadMarkerHelp(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_CAPABILITIES)) {
			loadCapabilities(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_WORKINGSETS)) {
			loadWorkingSets(ext);
			return;
		}
				
	}

	public void revoke(IExtensionPoint extPt, IExtension ext) {
		String name = extPt.getSimpleIdentifier();
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_VIEWS)) {
			unloadView(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_EDITOR)) {
			unloadEditor(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_NEW)) {
			NewWizardsRegistryReader.removeExtension(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_PERSPECTIVES)) {
			unloadPerspective(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_ACTION_SETS)) {
			unloadActionSets(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_ACTION_SET_PART_ASSOCIATIONS)) {
			unloadActionSetPartAssociation(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_MARKER_IMAGE_PROVIDER)) {
			unloadMarkerImageProvider(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_MARKER_HELP)) {
			unloadMarkerHelp(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_CAPABILITIES)) {
			unloadCapabilities(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_WORKINGSETS)) {
			unloadWorkingSets(ext);
			return;
		}

	}
	
	public void stop(IExtensionPoint extPt, IExtension ext) {
		String name = extPt.getSimpleIdentifier();
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_VIEWS)) {
			stopView(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_EDITOR)) {
			stopEditor(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_PERSPECTIVES)) {
			stopPerspective(ext);
			return;
		}
		if (name.equalsIgnoreCase(IWorkbenchConstants.PL_ACTION_SETS)) {
			stopActionSets(ext);
			return;
		}
	}
	
	private void loadView(IExtension ext) {
		MultiStatus result = new MultiStatus(
			PlatformUI.PLUGIN_ID,IStatus.OK,
			WorkbenchMessages.getString("Workbench.problemsRestoring"),null); //$NON-NLS-1$
		IViewRegistry vReg = WorkbenchPlugin.getDefault().getViewRegistry();
		ViewRegistryReader vReader = new ViewRegistryReader();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++) {
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			if (elements[i].getName().equals(IWorkbenchConstants.TAG_VIEW) && vReg.find(id)!=null)
				continue;
			if (elements[i].getName().equals(TAG_CATEGORY) && ((ViewRegistry)vReg).findCategory(id)!=null)
				continue;
			vReader.readElement((ViewRegistry)vReg, elements[i]);
		}
		restoreViewState(result);
		if (result.getSeverity() == IStatus.ERROR) {
			ErrorDialog.openError(
				null,
				WorkbenchMessages.getString("Workspace.problemsTitle"), //$NON-NLS-1$
				WorkbenchMessages.getString("Workbench.problemsRestoringMsg"), //$NON-NLS-1$
				result);
		}
	}

	private void restoreViewState(MultiStatus result){
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		IMemento memento;
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				memento = (IMemento) ((WorkbenchPage)pages[j]).getViewState();
				if (memento == null)
					continue;
				IMemento[] viewMems = memento.getChildren(IWorkbenchConstants.TAG_VIEW);
				ViewFactory viewFactory = ((WorkbenchPage)pages[j]).getViewFactory();
				for (int k = 0; k < viewMems.length; k++) {
					viewFactory.restoreViewState(viewMems[k]);
					createOpenPerspectiveView(pages[j], viewFactory, viewMems[k]);
				}
				((WorkbenchPage)pages[j]).setViewState(null);
			}
		}
	}

	private void createOpenPerspectiveView(IWorkbenchPage page, ViewFactory viewFactory, IMemento memento) {
		String id = memento.getString(IWorkbenchConstants.TAG_ID);
		String perspId = memento.getString(IWorkbenchConstants.TAG_PERSPECTIVE);
		Perspective persp = ((WorkbenchPage)page).getActivePerspective();
		if (persp.getDesc().getId().equals(perspId)) {
			try {
				viewFactory.createView(id);
				page.showView(id);
			} catch (PartInitException e) {}
		}
	}

	private void unloadView(IExtension ext) {
		final MultiStatus result = new MultiStatus(
			PlatformUI.PLUGIN_ID,IStatus.OK,
			WorkbenchMessages.getString("ViewFactory.problemsSavingViews"),null); //$NON-NLS-1$
		IViewRegistry vReg = WorkbenchPlugin.getDefault().getViewRegistry();
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				ArrayList viewsRemoved = new ArrayList();
				XMLMemento memento = XMLMemento.createWriteRoot(IWorkbenchConstants.TAG_VIEWS);
				((WorkbenchPage)pages[j]).setViewState(memento);
				IConfigurationElement[] elements = ext.getConfigurationElements();
				for(int k = 0; k < elements.length; k++) {
					if (!elements[k].getName().equals(IWorkbenchConstants.TAG_VIEW))
						continue;
					String id = elements[k].getAttribute(IWorkbenchConstants.TAG_ID);
					if (id != null) {
						ViewFactory viewFactory = ((WorkbenchPage)pages[j]).getViewFactory(); 
						IViewReference viewRef = viewFactory.getView(id);
						if (viewRef != null) {
							if (isViewOpen(viewRef, viewFactory)) 
								saveViewState(pages[j], id, viewFactory.saveViewState(memento, viewRef, result));
							((WorkbenchPage)pages[j]).hideView(viewRef);
							((WorkbenchPage)pages[j]).getViewFactory().releaseView(id);
						}
						viewsRemoved.add(id);
						((ViewRegistry)vReg).remove(id);
					}
				}
				Object[] showViewIdsRemoved = findShowViewIdsRemoved(
					((WorkbenchPage)pages[j]).getShowViewActionIds(), viewsRemoved);
				if (showViewIdsRemoved.length > 0)
					removeViewIdsFromShowViewMenu(window, showViewIdsRemoved);
			}
		}
		if (result.getSeverity() != IStatus.OK) {
			ErrorDialog.openError((Shell)null,
				WorkbenchMessages.getString("Workbench.problemsSaving"),  //$NON-NLS-1$
			WorkbenchMessages.getString("Workbench.problemsSavingMsg"), //$NON-NLS-1$
			result);
		}
	}
	
	private void saveViewState(IWorkbenchPage page, String id, IMemento memento) {
		Perspective persp = ((WorkbenchPage)page).getActivePerspective();
		if (persp.findView(id) != null)
			memento.putString(IWorkbenchConstants.TAG_PERSPECTIVE, persp.getDesc().getId());
	}

	private Object[] findShowViewIdsRemoved(ArrayList showViewIds, ArrayList viewsRemoved) {
		ArrayList list = new ArrayList();
		Object[] showViewIdList = showViewIds.toArray();
		Object[] viewsRemovedList = viewsRemoved.toArray();
		for(int i=0; i<showViewIdList.length; i++)
			for(int j=0; j<viewsRemovedList.length; j++)
				if (((String)viewsRemovedList[j]).equals((String)showViewIdList[i]))
					list.add(viewsRemovedList[j]);
		return list.toArray();
	}

	private void removeViewIdsFromShowViewMenu(IWorkbenchWindow window, Object[] viewsRemoved) {
		MenuManager menuManager = ((WorkbenchWindow)window).getMenuManager();
		IContributionItem[] items = menuManager.getItems();
		menuManager = null;
		for(int i=0; i<items.length; i++)
			if (items[i] instanceof MenuManager && ((MenuManager)items[i]).getMenuText().equals("&Window")) {
				menuManager = (MenuManager)items[i];
				break;
			}
		if (menuManager == null)
			return;
		items = menuManager.getItems();
		menuManager = null;
		for(int i=0; i<items.length; i++)
			if (items[i] instanceof MenuManager && ((MenuManager)items[i]).getMenuText().equals("Show &View")) {
				menuManager = (MenuManager)items[i];
				break;
			}
		if (menuManager == null)
			return;
		items = menuManager.getItems();
		if (items.length < 1 || !(items[0] instanceof ShowViewMenu))
			return;
		
		for(int i=0; i<viewsRemoved.length; i++)
			((ShowViewMenu)items[0]).removeAction((String)viewsRemoved[i]);
	}

	private boolean isViewOpen(IViewReference view, ViewFactory viewFactory) {
		IViewReference[] views = viewFactory.getViews();
		for(int i=0; i<views.length; i++)
			if (view == views[i])
				return true;
		return false;
	}

	private void loadEditor(IExtension ext) {
		MultiStatus result = new MultiStatus(
			PlatformUI.PLUGIN_ID,IStatus.OK,
			WorkbenchMessages.getString("Workbench.problemsRestoring"),null); //$NON-NLS-1$
		IEditorRegistry eReg = WorkbenchPlugin.getDefault().getEditorRegistry();
		EditorRegistryReader eReader = new EditorRegistryReader();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++) {
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			if (id != null && eReg.findEditor(id) != null)
				continue;
			eReader.readElement((EditorRegistry)eReg, elements[i]);
			restoreEditorState(elements[i], result);
		}
		if (result.getSeverity() == IStatus.ERROR) {
			ErrorDialog.openError(
				null,
				WorkbenchMessages.getString("Workspace.problemsTitle"), //$NON-NLS-1$
				WorkbenchMessages.getString("Workbench.problemsRestoringMsg"), //$NON-NLS-1$
				result);
		}
	}
	
	private void restoreEditorState(IConfigurationElement element, MultiStatus result){
		String id = element.getAttribute(IWorkbenchConstants.TAG_ID);
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		IMemento memento;
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				IEditorReference existingVisible = ((WorkbenchPage)pages[j]).getEditorPresentation().getVisibleEditor();
				EditorManager editorManager = ((WorkbenchPage)pages[j]).getEditorManager();
				ArrayList visibleEditors = new ArrayList(5);
				IEditorPart activeEditor[] = new IEditorPart[1];
				ArrayList errorWorkbooks = new ArrayList(1);
				memento = (IMemento) ((WorkbenchPage)pages[j]).getEditorStateMap().remove(id);
				if (memento == null)
					continue;
				IMemento[] editorMems = memento.getChildren(IWorkbenchConstants.TAG_EDITOR);
				for (int k = 0; k < editorMems.length; k++) {
					if (!checkOpenable(editorMems[k], pages[j]))
						continue;
					editorManager.restoreEditorState(
						editorMems[k], visibleEditors, activeEditor, errorWorkbooks, result);
				}
				if (existingVisible == null)
					for (int k = 0; k < visibleEditors.size(); k++)
						editorManager.setVisibleEditor((IEditorReference) visibleEditors.get(k), false);
				else
					editorManager.setVisibleEditor(existingVisible, true);
					
				if (visibleEditors.size() == 1)
				for (Iterator iter = errorWorkbooks.iterator(); iter.hasNext();) {
					iter.next();
					((WorkbenchPage)pages[j]).getEditorPresentation().fixVisibleEditor();
				}
			}
		}
	}
	
	private boolean checkOpenable(IMemento memento, IWorkbenchPage page) {
		IMemento inputMem = memento.getChild(IWorkbenchConstants.TAG_INPUT);
		String factoryID = null;
		if(inputMem != null)
			factoryID = inputMem.getString(IWorkbenchConstants.TAG_FACTORY_ID);
		if (factoryID == null)
			return false;
		IElementFactory factory = WorkbenchPlugin.getDefault().getElementFactory(factoryID);
		if (factory == null)
			return false;
		IAdaptable input = factory.createElement(inputMem);
		if (input == null || !(input instanceof IEditorInput))
			return false;
		IEditorInput editorInput = (IEditorInput) input;
		IEditorReference edRefs[] = page.getEditorReferences();
		
		for(int i=0; i<edRefs.length; i++) {
			IEditorPart editor = edRefs[i].getEditor(false);
			if (editor == null)
				continue;
			IEditorInput edInput = editor.getEditorInput();
			if (edInput.equals(editorInput))
				return false;
		}
		return true;
	}

	private void unloadEditor(IExtension ext) {
		MultiStatus result = new MultiStatus(
			PlatformUI.PLUGIN_ID,IStatus.OK,
			WorkbenchMessages.getString("EditorManager.problemsSavingEditors"),null); //$NON-NLS-1$
		EditorRegistry eReg = (EditorRegistry)WorkbenchPlugin.getDefault().getEditorRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for(int i=0; i<elements.length; i++) {
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			for (int j = 0; j < windows.length; j++) {
				WorkbenchWindow window = (WorkbenchWindow) windows[j];
				IWorkbenchPage[] pages = window.getPages();
				for (int k = 0; k < pages.length; k++)
					closeEditors(pages[k], id, result);
			}
			eReg.remove(id);
		}
		if (result.getSeverity() != IStatus.OK) {
			ErrorDialog.openError((Shell)null,
				WorkbenchMessages.getString("Workbench.problemsSaving"),  //$NON-NLS-1$
			WorkbenchMessages.getString("Workbench.problemsSavingMsg"), //$NON-NLS-1$
			result);
		}
	}
	
	private void closeEditors(IWorkbenchPage page, String id, MultiStatus result) {
		XMLMemento memento = XMLMemento.createWriteRoot(IWorkbenchConstants.TAG_EDITORS);
		IEditorReference[] editorRefs = page.getEditorReferences();
		boolean changed = false;
		for(int i = 0; i < editorRefs.length; i++) {
			if (editorRefs[i].getId().equals(id)) {
				IEditorPart editor = editorRefs[i].getEditor(true);
				EditorManager editorManager = ((WorkbenchPage)page).getEditorManager();
				if(editor == null) {
					IMemento mem = editorManager.getMemento(editorRefs[i]);
					if(mem != null) {
						IMemento editorMem = memento.createChild(IWorkbenchConstants.TAG_EDITOR);
						editorMem.putMemento(mem);
					}
				} else
					editorManager.saveEditorState(memento, editor, result);
				((WorkbenchPage)page).getEditorStateMap().put(id, memento);
				page.closeEditor(editor, true);
				changed = true;
			}
		}
	}	
	
	private void unloadPerspective(IExtension ext) {
		final MultiStatus result = new MultiStatus(
			PlatformUI.PLUGIN_ID,IStatus.OK,
			WorkbenchMessages.getString("ViewFactory.problemsSavingViews"),null); //$NON-NLS-1$
		IPerspectiveRegistry pReg = WorkbenchPlugin.getDefault().getPerspectiveRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i = 0; i < elements.length; i++) {
			if (!elements[i].getName().equals(IWorkbenchConstants.TAG_PERSPECTIVE))
				continue;
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			if (id == null)
				continue;
			IPerspectiveDescriptor desc = pReg.findPerspectiveWithId(id);
			if (desc == null)
				continue;
			((PerspectiveRegistry)pReg).deletePerspective(desc);
			IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
			for (int j = 0; j < windows.length; j++) {
				WorkbenchWindow window = (WorkbenchWindow) windows[j];
				IWorkbenchPage[] pages = window.getPages();
				for (int k = 0; k < pages.length; k++) {
					Perspective persp = ((WorkbenchPage)pages[k]).findPerspective(desc);
					if (persp == null)
						return;
					XMLMemento memento = (XMLMemento)((WorkbenchPage)pages[k]).getPerspectiveState();
					if (memento == null) {
						memento = XMLMemento.createWriteRoot(IWorkbenchConstants.TAG_PERSPECTIVES);
						((WorkbenchPage)pages[k]).setPerspectiveState(memento);
					}
					IMemento  childMem = memento.createChild(IWorkbenchConstants.TAG_PERSPECTIVE);
					result.merge(persp.saveState(childMem));
					((WorkbenchPage)pages[k]).closePerspective(desc, true);
					((Workbench)workbench).getPerspectiveHistory().removeItem(desc);
				}
			}
		}
		if (result.getSeverity() != IStatus.OK) {
			ErrorDialog.openError((Shell)null,
				WorkbenchMessages.getString("Workbench.problemsSaving"),  //$NON-NLS-1$
			WorkbenchMessages.getString("Workbench.problemsSavingMsg"), //$NON-NLS-1$
			result);
		}
	}

	private void loadPerspective(IExtension ext) {
		MultiStatus result = new MultiStatus(
			PlatformUI.PLUGIN_ID,IStatus.OK,
			WorkbenchMessages.getString("Workbench.problemsRestoring"),null); //$NON-NLS-1$
		IPerspectiveRegistry pReg = WorkbenchPlugin.getDefault().getPerspectiveRegistry();
		PerspectiveRegistryReader pReader = new PerspectiveRegistryReader((PerspectiveRegistry)pReg);
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++) {
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			if (id == null)
				continue;
			IPerspectiveDescriptor desc = pReg.findPerspectiveWithId(id);
			if (desc == null)
				pReader.readElement(elements[i]);
		}
		restorePerspectiveState(result);
		if (result.getSeverity() == IStatus.ERROR) {
			ErrorDialog.openError(
				null,
				WorkbenchMessages.getString("Workspace.problemsTitle"), //$NON-NLS-1$
				WorkbenchMessages.getString("Workbench.problemsRestoringMsg"), //$NON-NLS-1$
				result);
		}
	}

	private void restorePerspectiveState(MultiStatus result){
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		IMemento memento;
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				memento = (IMemento) ((WorkbenchPage)pages[j]).getPerspectiveState();
				if (memento == null)
					continue;
				IMemento[] perspMems = memento.getChildren(IWorkbenchConstants.TAG_PERSPECTIVE);
				for (int k = 0; k < perspMems.length; k++) {
					try {
						Perspective persp = new Perspective(null, ((WorkbenchPage)pages[j]));
						result.merge(persp.restoreState(perspMems[k]));
						((WorkbenchPage)pages[j]).addPerspective(persp);
						((WorkbenchWindow)windows[i]).addPerspectiveShortcut(persp.getDesc(), (WorkbenchPage)pages[j]);
					} catch (WorkbenchException e) {}
				}
				((WorkbenchPage)pages[j]).setPerspectiveState(null);
			}
		}
	}
	
	private void loadActionSets(IExtension ext) {
		ActionSetRegistry aReg = (ActionSetRegistry)WorkbenchPlugin.getDefault().getActionSetRegistry();
		ActionSetRegistryReader reader = new ActionSetRegistryReader(aReg);
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			reader.readElement(elements[i]);
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage page = window.getActivePage();
			Perspective persp = ((WorkbenchPage)page).getActivePerspective();
			if (persp == null)
				continue;
			for(int j = 0; j < elements.length; j++) {
				if (!elements[j].getName().equals(IWorkbenchConstants.TAG_ACTION_SET))
					continue;
				String id = elements[j].getAttribute(IWorkbenchConstants.TAG_ID);
				if (id != null) {
					IActionSetDescriptor desc =  aReg.findActionSet(id);
					if (desc != null) {
						persp.addActionSet(desc);
//						page.showActionSet(id);
					}
				}
			}
			window.updateActionSets();
		}
	}
	
	private void unloadActionSets(IExtension ext) {
		ActionSetRegistry aReg = (ActionSetRegistry)WorkbenchPlugin.getDefault().getActionSetRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				for(int k = 0; k < elements.length; k++) {
					if (!elements[k].getName().equals(IWorkbenchConstants.TAG_ACTION_SET))
						continue;
					String id = elements[k].getAttribute(IWorkbenchConstants.TAG_ID);
					if (id != null) {
						aReg.remove(id);
						removeActionSet((WorkbenchPage)pages[j], id);
					}
				}
			}
		}
	}

	private void removeActionSet(WorkbenchPage page, String id) {
		Perspective persp = page.getActivePerspective();
		ActionPresentation actionPresentation = ((WorkbenchWindow)page.getWorkbenchWindow()).getActionPresentation();
		IActionSet[] actionSets = actionPresentation.getActionSets();
		for(int i=0; i<actionSets.length; i++) {
			IActionSetDescriptor desc = ((PluginActionSet)actionSets[i]).getDesc();
			if (id.equals(desc.getId())) {
				PluginActionSetBuilder builder = new PluginActionSetBuilder();
				builder.removeActionExtensions((PluginActionSet)actionSets[i], page.getWorkbenchWindow());
				actionPresentation.removeActionSet(desc);
			}
		}
		if (persp != null)
			persp.removeActionSet(id);
	}

	private void loadActionSetPartAssociation(IExtension ext) {
		ActionSetRegistry aReg = (ActionSetRegistry)WorkbenchPlugin.getDefault().getActionSetRegistry();
		ActionSetPartAssociationsReader reader = new ActionSetPartAssociationsReader(aReg);
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			reader.readElement(elements[i]);
	}
	
	private void unloadActionSetPartAssociation(IExtension ext) {
		ActionSetRegistry aReg = (ActionSetRegistry)WorkbenchPlugin.getDefault().getActionSetRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i = 0; i < elements.length; i++) {
			String type = elements[i].getName();
			if (!type.equals(IWorkbenchConstants.PL_ACTION_SET_PART_ASSOCIATIONS))
				continue;
			String actionSetId = elements[i].getAttribute(ATT_TARGET_ID);
			IConfigurationElement [] children = elements[i].getChildren();
			for (int j = 0; j < children.length; j++) {
				IConfigurationElement child = children[j];
				type = child.getName();
				if (type.equals(TAG_PART)) {
					String partId = child.getAttribute(ATT_ID);
					if (partId != null) 
					aReg.removeAssociation(actionSetId, partId);
				}
			}
		}
	}
	private void loadMarkerImageProvider(IExtension ext) {
		MarkerImageProviderRegistry mReg = (MarkerImageProviderRegistry)WorkbenchPlugin.getDefault().getMarkerImageProviderRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			if (elements[i].getName().equals(TAG_PROVIDER)) 
				mReg.addProvider(elements[i]);
	}
	
	private void unloadMarkerImageProvider(IExtension ext) {
		MarkerImageProviderRegistry mReg = (MarkerImageProviderRegistry)WorkbenchPlugin.getDefault().getMarkerImageProviderRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			if (elements[i].getName().equals(TAG_PROVIDER)) 
				mReg.removeProvider(elements[i]);
	}

	private void loadMarkerHelp(IExtension ext) {
		MarkerHelpRegistry mReg = (MarkerHelpRegistry)WorkbenchPlugin.getDefault().getMarkerHelpRegistry();
		MarkerHelpRegistryReader reader = new MarkerHelpRegistryReader(mReg);		
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			reader.readElement(elements[i]);
	}
	
	private void unloadMarkerHelp(IExtension ext) {
		MarkerHelpRegistry mReg = (MarkerHelpRegistry)WorkbenchPlugin.getDefault().getMarkerHelpRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			mReg.removeElement(elements[i].getAttribute(IWorkbenchConstants.TAG_ID));
	}	

	private void loadCapabilities(IExtension ext) {
		CapabilityRegistry cReg = (CapabilityRegistry)WorkbenchPlugin.getDefault().getCapabilityRegistry();
		CapabilityRegistryReader reader = new CapabilityRegistryReader(cReg);		
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			reader.readElement(elements[i]);
	}
	
	private void unloadCapabilities(IExtension ext) {
		CapabilityRegistry cReg = (CapabilityRegistry)WorkbenchPlugin.getDefault().getCapabilityRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			cReg.removeElement(elements[i].getAttribute(IWorkbenchConstants.TAG_ID));
	}	

	private void loadWorkingSets(IExtension ext) {
		WorkingSetRegistry wReg = (WorkingSetRegistry)WorkbenchPlugin.getDefault().getWorkingSetRegistry();
		WorkingSetRegistryReader reader = new WorkingSetRegistryReader(wReg);		
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			reader.readElement(elements[i]);
	}
	
	private void unloadWorkingSets(IExtension ext) {
		WorkingSetRegistry wReg = (WorkingSetRegistry)WorkbenchPlugin.getDefault().getWorkingSetRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i=0; i<elements.length; i++)
			wReg.removeWorkingSetDescriptor(elements[i].getAttribute(IWorkbenchConstants.TAG_ID));
	}

	private void stopView(IExtension ext) {
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				IConfigurationElement[] elements = ext.getConfigurationElements();
				for(int k = 0; k < elements.length; k++) {
					if (!elements[k].getName().equals(IWorkbenchConstants.TAG_VIEW))
						continue;
					String id = elements[k].getAttribute(IWorkbenchConstants.TAG_ID);
					if (id != null) {
						ViewFactory viewFactory = ((WorkbenchPage)pages[j]).getViewFactory(); 
						IViewReference viewRef = viewFactory.getView(id);
						if (viewRef != null) {
							((WorkbenchPage)pages[j]).hideView(viewRef);
							((WorkbenchPage)pages[j]).getViewFactory().releaseView(id);
						}
					}
				}
			}
		}
	}
	private void stopEditor(IExtension ext) {
		IConfigurationElement[] elements = ext.getConfigurationElements();
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for(int i=0; i<elements.length; i++) {
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			for (int j = 0; j < windows.length; j++) {
				WorkbenchWindow window = (WorkbenchWindow) windows[j];
				IWorkbenchPage[] pages = window.getPages();
				for (int k = 0; k < pages.length; k++) {
					IEditorReference[] editorRefs = pages[k].getEditorReferences();
					for(int l = 0; l < editorRefs.length; l++) 
						if (editorRefs[l].getId().equals(id)) {
							IEditorPart editor = editorRefs[l].getEditor(true);
							if (editor != null)
								pages[k].closeEditor(editor, true);
						}
				}
			}
		}
	}
	private void stopPerspective(IExtension ext) {
		IPerspectiveRegistry pReg = WorkbenchPlugin.getDefault().getPerspectiveRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		for(int i = 0; i < elements.length; i++) {
			if (!elements[i].getName().equals(IWorkbenchConstants.TAG_PERSPECTIVE))
				continue;
			String id = elements[i].getAttribute(IWorkbenchConstants.TAG_ID);
			if (id == null)
				continue;
			IPerspectiveDescriptor desc = pReg.findPerspectiveWithId(id);
			if (desc == null)
				continue;
			((PerspectiveRegistry)pReg).deletePerspective(desc);
			IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
			for (int j = 0; j < windows.length; j++) {
				WorkbenchWindow window = (WorkbenchWindow) windows[j];
				IWorkbenchPage[] pages = window.getPages();
				for (int k = 0; k < pages.length; k++) {
					Perspective persp = ((WorkbenchPage)pages[k]).findPerspective(desc);
					if (persp == null)
						return;
					((WorkbenchPage)pages[k]).closePerspective(desc, true);
				}
			}
		}
	}
	private void stopActionSets(IExtension ext) {
		ActionSetRegistry aReg = (ActionSetRegistry)WorkbenchPlugin.getDefault().getActionSetRegistry();
		IConfigurationElement[] elements = ext.getConfigurationElements();
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; i < windows.length; i++) {
			WorkbenchWindow window = (WorkbenchWindow) windows[i];
			IWorkbenchPage[] pages = window.getPages();
			for (int j = 0; j < pages.length; j++) {
				for(int k = 0; k < elements.length; k++) {
					if (!elements[k].getName().equals(IWorkbenchConstants.TAG_ACTION_SET))
						continue;
					String id = elements[k].getAttribute(IWorkbenchConstants.TAG_ID);
					if (id != null) 
						((WorkbenchPage)pages[j]).hideActionSet(id);
				}
			}
		}
	}
}
